/*
                           Copyright (C) 1995 
             John R.Callahan, Sudhakar Ramakrishnan, Sun Wei.
	         Web Integrated Software Project 
                   Management and Metrics Tool
                           (WISE)

WISE [both binary and source] (hereafter, Software) is 
copyrighted by John R.Callahan, Sudhakar Ramakrishnan , Sun Wei and 
ownership remains with the authors.

The authors grant you (hereafter, Licensee), a license to use the Software
for academic, research and internal business purposes only, without a
fee.  Licensee may distribute the binary to third parties providing 
that the copyright notice and this statement appears on all copies and
that no charge is associated with such copies. 

Licensee may make derivative works.  However, if Licensee distributes
any derivative work based on or derived from the Software, then
Licensee will (1) notify the authors regarding its distribution of 
the derivative work, and (2) clearly notify users that such derivative 
work is a modified version and not the original WISE tool distributed 
by the authors.

Any Licensee wishing to make commercial use of the Software should
contact the authors to negotiate an appropriate license for such
commercial use.  Commercial use includes (1) integration of all or
part of the source code into a product for sale or license by or on
behalf of licensee to third parties, or (2) distribution of the binary
code or source code to third parties that need it to utilize a
commercial product sold or licensed by or on behalf of Licensee.

THE AUTHORS MAKE NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS 
SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR 
IMPLIED WARRANTY.  THE AUTHORS SHALL NOT BE LIABLE FOR ANY DAMAGES
SUFFERED BY USERS OF THIS SOFTWARE.

By using or copying this Software, Licensee agrees to abide by the
copyright law and all other applicable laws of the U.S., including but
not limited, to, export control laws, and the terms of this license.
The authors shall have the right to terminate this license immediately by
written notice upon Licensee's breach of, or non-compliance with, any
of its terms.  Licensee may be held legally responsible for any
copyright infringement that is caused, directly or indirectly, by
Licensee's failure to abide by the terms of this license.

Direct notices to:
	
	John R.Callahan 
	886 Chestnut Ridge Road
	Concurrent Engineering Research Center
	West Virginia University
	Morgantown, WV 26505
	USA

	phone:	(304) 293-7226
	email:	callahan@cerc.wvu.edu
*/

#define WISEMAIN 1

#include <stdio.h>
#include <time.h>
#include "msql.h"
#include "wise.h"

#define HASHMAX 100

struct item {
  char *str;
  int count;
  struct item *next;
} *head = NULL;
struct hashitem {
  int issueno;
  char* status;
  struct hashitem *next;
} *hashtable[HASHMAX];

void
resetcount(int max)
{
  int i;
  struct item *p;

  for(i=0;i < HASHMAX;i++) {
    hashtable[i] = NULL;
  }
  head = NULL;
  for(i=0;i < max;i++) {
    p = (struct item *)malloc(sizeof(struct item));
    p->str = NULL;
    p->count = 0;
    p->next = head;
    head = p;
  }
  return;
}

void
addcount(char* str,int issueno)
{
  int hashkey;
  struct item *p;
  struct hashitem *hp;
  char *old = NULL;

  hashkey = issueno % HASHMAX;
  for(hp=hashtable[hashkey];hp != NULL;hp=hp->next) {
    if(hp->issueno == issueno) {
      old = hp->status;
      hp->status = str;
      break;
    }
  }
  if(hp == NULL) {
    hp = (struct hashitem *)malloc(sizeof(struct hashitem));
    hp->issueno = issueno;
    hp->status = str;
    hp->next = hashtable[hashkey];
    hashtable[hashkey] = hp;
  }
  for(p=head;p != NULL;p=p->next) {
    if(p->str == NULL) {
      p->str = str;
      p->count = 1;
      break;
    } else if(strcmp(p->str,str) == 0) {
      p->count++;
      break;
    }
  }
  for(p=head;old != NULL && p != NULL;p=p->next) {
    if(p->str != NULL && strcmp(p->str,old) == 0) {
      p->count--;
    }
  }
  return;
}

main()
{
  int sock;
  int i,j,k=1,first;
  char tbuf[WISEBUFMAX];
  char dbuf[WISEBUFMAX];
  struct item *p;
  m_result *result;
  m_row *row;
  time_t ts1970;
  struct tm *pstm;
  FILE *f,*pf;
  char* db;
  char* dbhost;

  ts1970 = time(NULL);
  /* cftime(dbuf,"%Y-%m-%d %T %Z",&ts1970); */
  pstm = localtime (&ts1970);
  strftime(dbuf,WISEBUFMAX,"%Y-%m-%d %T %Z",pstm);

  for(i=0;wiseproject[i].projectname != NULL;i++) {

    if((dbhost = wise_getprojhost(wiseproject[i].projectname)) == NULL) {
      printf("<H1>Could not find project database host!</H1>");
      return;
    }

    wise_setprojport(wiseproject[i].projectname);
    if((sock = msqlConnect(dbhost)) < 0) {
      printf("<H1>Connect error: %s</H1>",msqlErrMsg);
      return;
    }

    if((db = wise_getprojdb(wiseproject[i].projectname)) == NULL) {
      printf("<H1>Could not get project's database name!</H1>");
      return;
    }

    if(msqlSelectDB(sock,db) < 0) {
      fprintf(stderr,"Cannot connect to database %s!\n",db);
      return -1;
    }

    sprintf(tbuf,"%s/%s.metrics",WISE_METRICDIR,
	    wiseproject[i].projectname);
    if((pf = fopen(tbuf,"w+")) == NULL) {
      fprintf(stderr,"Cannot create file %s!\n",tbuf);
      return -1;
    }
    fprintf(pf,"Metrics for %s\n",
	    wise_getprojlabel(wiseproject[i].projectname));
    fclose(pf);

    sprintf(tbuf,"%s/%s-metrics-tsum.html",WISE_METRICDIR,
	    wiseproject[i].projectname);
    if((pf = fopen(tbuf,"w+")) == NULL) {
      fprintf(stderr,"Cannot create file %s!\n",tbuf);
      return -1;
    }
    fprintf(pf,"<!DOCTYPE HTML SYSTEM \"html.dtd\">\n");
    fprintf(pf,"<HTML>\n");
    fprintf(pf,"<HEAD>\n");
    fprintf(pf,"<TITLE>Project %s Metrics</TITLE>\n",
	    wise_getprojlabel(wiseproject[i].projectname));
    fprintf(pf,"</HEAD>\n");
    fprintf(pf,"<BODY BACKGROUND=\"%s\"",WISEBKGND);
    fprintf(pf,"TEXT=\"000000\"");
    fprintf(pf,"LINK=\"0000ff\" VLINK=\"0000ff\">\n");
    fprintf(pf,"<BASEFONT SIZE=4>\n");
    fprintf(pf,"<center>\n");
    fprintf(pf,"<table border=1 cellpadding=1>\
        <tr><td rowspan=4><table border=5 cellpadding=5><tr><td>\
        <center><IMG SRC=\"%s\" align=center><BR>A WISE application<BR>&copy 1995 <A HREF=\"http://research.ivv.nasa.gov/\">West Virginia University</A></center></td></tr></table></td></tr>\
        <tr><td><H2>%s Metrics</H2></td></tr>\
        <tr><td><H2>as of %s</H2></td></tr>\
        </table></center><BR><HR>\
        ",wise_getprojimage(wiseproject[i].projectname),wise_getprojlabel(wiseproject[i].projectname),dbuf);

    for(j=0;wiseproject[i].fields[j].field != NULL;j++) {

      if((wiseproject[i].fields[j].fieldtype == WISE_SELECT)
	 || (wiseproject[i].fields[j].fieldtype == WISE_RADIO)
	 || (wiseproject[i].fields[j].fieldtype == WISE_CBOX)) {

	fprintf(pf,"<A HREF=\"http:/%s%s-%s-tsum.html\">",
		WISEMETRICS,wiseproject[i].projectname,
		wiseproject[i].fields[j].field);
	fprintf(pf,"<IMG SRC=\"http:/%stsumS-%s-%s.gif\"></A><BR>",
		WISEMETRICS,wiseproject[i].projectname,
		wiseproject[i].fields[j].field);

	sprintf(tbuf,"select distinct %s from %slog where %s <> ''",
		wiseproject[i].fields[j].field,
		wiseproject[i].projectname,
		wiseproject[i].fields[j].field);

	if(msqlQuery(sock,tbuf) < 0) {
	  printf("<H1>Query error: %s</H1>",msqlErrMsg);
	  return -1;
	}
	result = msqlStoreResult();
	resetcount(msqlNumRows(result));
	msqlFreeResult(result);

	sprintf(tbuf,"select clock,%s,issueno from %slog where %s <> '' order by clock",
		wiseproject[i].fields[j].field,
		wiseproject[i].projectname,
		wiseproject[i].fields[j].field);

	if(msqlQuery(sock,tbuf) < 0) {
	  printf("<H1>Query error: %s</H1>",msqlErrMsg);
	  return -1;
	}
	result = msqlStoreResult();

	sprintf(tbuf,"%s/tsum-%s-%s.dat",WISE_METRICDIR,
		wiseproject[i].projectname,wiseproject[i].fields[j].field);
	if((f = fopen(tbuf,"w+")) == NULL) {
	  fprintf(stderr,"Cannot create file %s!\n",tbuf);
	  return -1;
	}
	first = 1;
	while((row = msqlFetchRow(result)) != NULL) {
	  if(first) {
	    fprintf(f,"%s",row[0]);
	    for(p=head;p != NULL;p=p->next) {
	      fprintf(f," %d",p->count);
	    }
	    fprintf(f,"\n");
	    first = 0;
	  }
	  addcount(row[1],atoi(row[2]));
	  fprintf(f,"%s",row[0]);
	  for(p=head;p != NULL;p=p->next) {
	    fprintf(f," %d",p->count);
	  }
	  fprintf(f,"\n");
	}
	fclose(f);

	sprintf(tbuf,"%s/%s-%s-tsum.html",
	  WISE_METRICDIR,wiseproject[i].projectname,
          wiseproject[i].fields[j].field);
	if((f=fopen(tbuf,"w+")) == NULL) {
	  fprintf(stderr,"Cannot create file %s!\n",tbuf);
	  return -1;
	}
	fprintf(f,"<!DOCTYPE HTML SYSTEM \"html.dtd\">\n");
	fprintf(f,"<HTML>\n");
	fprintf(f,"<HEAD>\n");
	fprintf(f,"<TITLE>Project %s Metrics for %s (as of %s)</TITLE>\n",
		wise_getprojlabel(wiseproject[i].projectname),
		wiseproject[i].fields[j].field,dbuf);
	fprintf(f,"</HEAD>\n");
	fprintf(f,"<BODY BACKGROUND=\"%s\"",WISEBKGND);
	fprintf(f,"TEXT=\"000000\"");
	fprintf(f,"LINK=\"0000ff\" VLINK=\"0000ff\">\n");
	fprintf(f,"<BASEFONT SIZE=4>\n");
	fprintf(f,"<table border=1 cellpadding=1>\
          <tr><td rowspan=4><table border=5 cellpadding=5><tr><td>\
          <center><IMG SRC=\"%s\" align=center><BR>A WISE application<BR>&copy 1995 <A HREF=\"http://research.ivv.nasa.gov/\">West Virginia University</A></center></td></tr></table></td></tr>\
          <tr><td><H2>%s Metrics for %s</H2></td></tr>\
          <tr><td><H2>as of %s</H2></td></tr>\
          </table></center><HR>\
          ",wise_getprojimage(wiseproject[i].projectname),
		wise_getprojlabel(wiseproject[i].projectname),
		wiseproject[i].fields[j].field,
		dbuf);

	fprintf(f,"<A HREF=\"http:/%stsumL-%s-%s.gif\"><IMG SRC=\"http:/%stsumL-%s-%s.gif\"></A><BR>",
		WISEMETRICS,
		wiseproject[i].projectname,
		wiseproject[i].fields[j].field,
		WISEMETRICS,
		wiseproject[i].projectname,
		wiseproject[i].fields[j].field);
	fprintf(f,"<center>\n");
	fclose(f);

	sprintf(tbuf,"%s/tsum.%s.metric",
	  WISE_METRICDIR,wiseproject[i].projectname);
	if((f=fopen(tbuf,"w+")) == NULL) {
	  fprintf(stderr,"Cannot create file %s!\n",tbuf);
	  return -1;
	}
	fprintf(f,"Time summary metrics for %s\n",
		wise_getprojlabel(wiseproject[i].projectname));
	fclose(f);
	
	/* big picture */
	sprintf(tbuf,"%s/tsumL-%s-%s.gp",WISE_METRICDIR,
		wiseproject[i].projectname,wiseproject[i].fields[j].field);
	if((f = fopen(tbuf,"w+")) == NULL) {
	  fprintf(stderr,"Cannot create file %s!\n",tbuf);
	  return -1;
	}
	fprintf(f,"set term pbm color\n");
	fprintf(f,"cd '%s'\n",WISE_METRICDIR);
	fprintf(f,"set output \"tsumL-%s-%s.pbm\"\n",
		wiseproject[i].projectname,wiseproject[i].fields[j].field);
	fprintf(f,"set data style steps\n");
	fprintf(f,"set size 0.9, 0.45\n");
	fprintf(f,"set xlabel 'Date\\nTime'\n");
	fprintf(f,"set timefmt \"%%Y-%%m-%%d %%H:%%M:%%S\"\n"); 
	fprintf(f,"set yrange [ 0 : ]\n");
	fprintf(f,"set xdata time\n"); 
	fprintf(f,"set title \"\"\n");
	fprintf(f,"set ylabel \"Number of Issues\"\n");
	fprintf(f,"set format x \"%%m/%%d\"\n");
	fprintf(f,"set key left\n"); 
/*	fprintf(f,"set grid\n"); */
	k = 2;
	for(p=head;p != NULL;p=p->next) {
	  if(p == head) {
	    fprintf(f,"plot ");
	  } else {
	    fprintf(f," \\\n, ");
	  }
	  fprintf(f,"'%s/tsum-%s-%s.dat'",WISE_METRICDIR,
		  wiseproject[i].projectname,wiseproject[i].fields[j].field);

          /* hack alert -- the version of gnuplot that I am using
           * is having some sort of difficulty with the data format.
           * as a result, the column numbering in the data files
           * is off by one.  This is a quick hack to work around
           * that, but this could bit the big one when a different
           * version of gnuplot is used. -- linas Janauary 1997 
	   * fprintf(f," using 1:%d t '%s'",k++,p->str);
           */
      
	  fprintf(f," using 1:%d t '%s'",++k,p->str);
	}
	fprintf(f,"\n");
	fclose(f);

	/* small picture */
	sprintf(tbuf,"%s/tsumS-%s-%s.gp",WISE_METRICDIR,
		wiseproject[i].projectname,wiseproject[i].fields[j].field);
	if((f = fopen(tbuf,"w+")) == NULL) {
	  fprintf(stderr,"Cannot create file %s!\n",tbuf);
	  return -1;
	}
	fprintf(f,"set term pbm color\n");
	fprintf(f,"cd '%s'\n",WISE_METRICDIR);
	fprintf(f,"set output \"tsumS-%s-%s.pbm\"\n",
		wiseproject[i].projectname,wiseproject[i].fields[j].field);
	fprintf(f,"set data style steps\n");
	fprintf(f,"set size 0.3, 0.3\n");
	fprintf(f,"set xlabel 'Date\\nTime'\n");
	fprintf(f,"set timefmt \"%%Y-%%m-%%d %%H:%%M:%%S\"\n"); 
	fprintf(f,"set yrange [ 0 : ]\n");
	fprintf(f,"set xdata time\n"); 
	fprintf(f,"set title \"\"\n");
	fprintf(f,"set ylabel \"Number of Issues\"\n");
	fprintf(f,"set format x \"%%m/%%d\"\n");
	fprintf(f,"set key left\n"); 
/*	fprintf(f,"set grid\n"); */
	k = 2;
	for(p=head;p != NULL;p=p->next) {
	  if(p == head) {
	    fprintf(f,"plot ");
	  } else {
	    fprintf(f," \\\n, ");
	  }
	  fprintf(f,"'%s/tsum-%s-%s.dat'",WISE_METRICDIR,
		  wiseproject[i].projectname,wiseproject[i].fields[j].field);

          /* hack alert -- the version of gnuplot that I am using
           * is having some sort of difficulty with the data format.
           * as a result, the column numbering in the data files
           * is off by one.  This is a quick hack to work around
           * that, but this could bit the big one when a different
           * version of gnuplot is used. -- linas Janauary 1997 
	   * fprintf(f," using 1:%d t '%s'",k++,p->str);
           */
      
	  fprintf(f," using 1:%d t '%s'",++k,p->str);
	}
	fprintf(f,"\n");
	fclose(f);

	msqlFreeResult(result);
      }
    }
    fprintf(pf,"</center>");
    fclose(pf);
    msqlClose(sock);
  }
  return 0;
}


